/*
 * export.c
 *
 * by Doug Wolfe
 * doug@wolfescience.com 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,  59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
 *
 */
 
#include <gtk/gtk.h>
#include "interface.h"
#include "export.h"
#include "support.h"
#include "callbacks.h"
#include "htmllite/htmlbuffer.h"

//for xml
#include <libxml/xmlreader.h>

//for transformation
#include <string.h>
#include <libxslt/transform.h>
#include <libxslt/xsltutils.h>


static int export_is_directory(char *filename)
{
	size_t len= strlen(filename);
	int ret;

	if (filename[len-1]== '\\')
	{
		filename[len-1]= '\0';
		ret= g_file_test(filename,G_FILE_TEST_IS_DIR);
		filename[len-1]= '\\';
	}
	else
		ret= g_file_test(filename,G_FILE_TEST_IS_DIR);

	return ret;
}

//directory has to end with a backslash
static void export_get_all_filenames(const char *directory, GArray *fileList)
{
	GDir *dir;
	const char* str;

	if ( !(dir= g_dir_open(directory, 0, NULL)) )
	{
		fprintf(stderr, "Cannot open directory %s\n", directory);
		return;
	}

	while (str= g_dir_read_name(dir) )
	{
		char *new_dir= g_strconcat(directory,str,"\\",NULL);

		if (export_is_directory(new_dir))
		{
			export_get_all_filenames(new_dir, fileList);
		}
		else
		{
			char *fn= g_strconcat(directory,str, NULL);
			g_array_append_val(fileList,fn);
		}

		g_free(new_dir);
	}
}

static void export_process_XSLT_Node(xmlTextReaderPtr reader, GtkTreeModel *store, GtkTreeIter iter)
{
	const xmlChar *name, *value;

	name = xmlTextReaderConstName(reader);
	if (name == NULL)
		name = BAD_CAST "--";

	value = xmlTextReaderConstValue(reader);

	if(!xmlStrcasecmp(name, (const xmlChar*)"xs:template-name") && xmlTextReaderNodeType(reader)== XML_READER_TYPE_ELEMENT  )
	{
		xmlTextReaderRead(reader);
		value = xmlTextReaderConstValue(reader);
		printf("gotcha:%s\n",value);
		gtk_tree_store_set (GTK_TREE_STORE(store), &iter,
				    0, value,
				    -1);
	}
	if(!xmlStrcasecmp(name, (const xmlChar*)"xs:template-description") && xmlTextReaderNodeType(reader)== XML_READER_TYPE_ELEMENT  )
	{
		xmlTextReaderRead(reader);
		value = xmlTextReaderConstValue(reader);
		printf("gotcha:%s\n",value);
		gtk_tree_store_set (GTK_TREE_STORE(store), &iter,
				    2, value,
				    -1);
	}
	if(!xmlStrcasecmp(name, (const xmlChar*)"xs:template-image") && xmlTextReaderNodeType(reader)== XML_READER_TYPE_ELEMENT  )
	{
		xmlTextReaderRead(reader);
		value = xmlTextReaderConstValue(reader);
		printf("gotcha:%s\n",value);
		gtk_tree_store_set (GTK_TREE_STORE(store), &iter,
				    3, value,
				    -1);
	}
	printf("%d %d %s %d %d",
	       xmlTextReaderDepth(reader),
	       xmlTextReaderNodeType(reader),
	       name,
	       xmlTextReaderIsEmptyElement(reader),
	       xmlTextReaderHasValue(reader));
	if (value == NULL)
		printf("\n");
	else {
		if (xmlStrlen(value) > 40)
			printf(" %.40s...\n", value);
		else
			printf(" %s\n", value);
	}
}

static void export_read_XSLT_File(const char *filename, GtkTreeModel *store)
{
	xmlTextReaderPtr reader;
	int ret;

	reader = xmlReaderForFile(filename, NULL, 0);
	if (reader != NULL) {
		GtkTreeIter   iter;
		gtk_tree_store_append (GTK_TREE_STORE(store), &iter, NULL);  /* Acquire an iterator */
		gtk_tree_store_set (GTK_TREE_STORE(store), &iter,
				    1, filename,
				    -1);
		
		ret = xmlTextReaderRead(reader);
		while (ret == 1) {
			export_process_XSLT_Node(reader, store, iter);
			ret = xmlTextReaderRead(reader);
		}
		xmlFreeTextReader(reader);
		if (ret != 0) {
			fprintf(stderr, "%s : failed to parse\n", filename);
		}
	} else {
		fprintf(stderr, "Unable to open %s\n", filename);
	}
}

static void
export_setup_XSLT_treeview (GtkWidget *treeview1)
{

	//create new treeview1 store and columns
	GtkTreeStore *store = gtk_tree_store_new (4,
			G_TYPE_STRING,
			G_TYPE_STRING,
			G_TYPE_STRING,
			G_TYPE_STRING); //name, filename, description, picture
	gtk_tree_view_set_model(GTK_TREE_VIEW(treeview1), GTK_TREE_MODEL(store));
	
	GtkCellRenderer *NameRender = gtk_cell_renderer_text_new ();
	g_object_set_data (G_OBJECT (NameRender), "column", 0);

	GtkTreeViewColumn *NameCol = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title(NameCol, "Templates");
	gtk_tree_view_column_pack_start (NameCol, NameRender, TRUE);
	gtk_tree_view_column_add_attribute (NameCol, NameRender, "text", 0);
	gtk_tree_view_column_add_attribute (NameCol, NameRender, "text", 1);
	gtk_tree_view_column_add_attribute (NameCol, NameRender, "text", 2);
	gtk_tree_view_column_add_attribute (NameCol, NameRender, "text", 3);
	gtk_tree_view_column_set_expand (NameCol, FALSE);
	gtk_tree_view_column_set_resizable(NameCol, TRUE);
	gtk_tree_view_column_set_sort_column_id(NameCol, 0);
	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview1), NameCol);

	gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(treeview1), TRUE);
}

extern int xmlLoadExtDtdDefaultValue;

static int export_transform(xmlDocPtr doc, char *xsltFileName, char *OutputFileName)
{
	int i;
	const char *params[16 + 1];
	int nbparams = 0;
	xsltStylesheetPtr cur = NULL;
	xmlDocPtr res;

	params[nbparams] = NULL;
	xmlSubstituteEntitiesDefault(1);
	xmlLoadExtDtdDefaultValue = 1;
	cur = xsltParseStylesheetFile((const xmlChar *)xsltFileName);
	i++;
	
	//doc = xmlParseFile(xmlFileName);
	res = xsltApplyStylesheet(cur, doc, params);
	xsltSaveResultToFile(stdout, res, cur);
	xsltSaveResultToFilename(OutputFileName, res, cur, 0);
	xsltFreeStylesheet(cur);
	xmlFreeDoc(res);
	xmlFreeDoc(doc);

	xsltCleanupGlobals();
	xmlCleanupParser();
	return(0);
}


GtkWidget * export_XSLT_dialog;
void export_dialog (GtkWidget *widget)
{
	if(export_XSLT_dialog == NULL)
	{
		export_XSLT_dialog = create_DialogExport ();
		GtkWidget *treeview1 = lookup_widget(export_XSLT_dialog, "treeview_Template");
		export_setup_XSLT_treeview(treeview1);
		GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview1));
	
		GtkTextView *textview_description = GTK_TEXT_VIEW(lookup_widget(GTK_WIDGET(export_XSLT_dialog), "textview_Description"));
		GtkTextBuffer * textbuffer = create_buffer(textview_description, FALSE);
        //for templates
        #ifdef G_OS_WIN32
               gchar *package_executable_dir = g_win32_get_package_installation_directory (NULL, NULL);
        #else
               gchar *package_executable_dir = g_build_filename (PACKAGE_DATA_DIR, PACKAGE, NULL);
        #endif
		const gchar *export_dir_string;
		export_dir_string = g_build_filename (package_executable_dir, "export/", NULL);
		printf("%s\n", export_dir_string);
		GArray *file_list;
		file_list= g_array_new(0,1,sizeof(char *));
		export_get_all_filenames(export_dir_string, file_list);
	
	
		int i;
		for (i=0; (guint)i < file_list->len; i++) {
			char *fn= g_array_index(file_list, char *, i);
			printf("+ %s\n", fn);
			export_read_XSLT_File(fn, store);
	
			g_free(fn);
		}
	
		g_signal_connect ((gpointer) treeview1, "cursor_changed",
					G_CALLBACK (on_treeview_Template_cursor_changed),
					NULL);
		g_array_free(file_list, 0);
	}
	if (gtk_dialog_run (GTK_DIALOG (export_XSLT_dialog)) == GTK_RESPONSE_OK)
	{
		printf("works\n");
		gtk_widget_hide(export_XSLT_dialog);
		GtkWidget *treeview1 = lookup_widget(export_XSLT_dialog, "treeview_Template");
        GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview1));
		GtkTreeSelection * selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview1));
		GtkTreeIter iter_file;
		if(gtk_tree_selection_get_selected (selection, &store, &iter_file))
		{
		//Fill Description box
			gchar * xslt_filename;
			gtk_tree_model_get (GTK_TREE_MODEL(store), &iter_file, 1, &xslt_filename, -1);
			if(xslt_filename!= NULL)
			{
				GtkWidget *dialog;
				dialog = create_filechooserdialog1 ();
				GtkWidget *btnOK = lookup_widget(GTK_WIDGET(dialog), "btnOK");
				gtk_button_set_label (GTK_BUTTON(btnOK), "gtk-save");
				gtk_button_set_use_stock (GTK_BUTTON(btnOK), TRUE);
				gtk_file_chooser_set_action(GTK_FILE_CHOOSER (dialog), GTK_FILE_CHOOSER_ACTION_SAVE);
				gtk_window_set_title(GTK_WINDOW(dialog), "Save File");

				if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
				{
					char * export_filename;
					export_filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));

					GtkWidget *treeview_xml = lookup_widget(widget, "treeview1");
					xmlDocPtr doc = WriteDoc_as_XHTML(treeview_xml);

					export_transform(doc,xslt_filename,export_filename);
					g_free(export_filename);
				}
				gtk_widget_destroy (dialog);
			}
		}
	}
	gtk_widget_hide(export_XSLT_dialog);
	
}
		
void on_treeview_Template_cursor_changed (GtkTreeView *treeview, gpointer user_data)
{
	//GtkWidget *treeview1 = lookup_widget(GTK_WIDGET(toolbutton), "treeview1");
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
	GtkTreeSelection * selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	GtkTreeIter iter;
	if(gtk_tree_selection_get_selected (selection, &store, &iter))
	{
		//Set export button sensitive
		GtkWidget *btn_export = lookup_widget(GTK_WIDGET(treeview), "btn_export");
		gtk_widget_set_sensitive (btn_export, TRUE);
		//Fill Description box
		gchar * description;
		printf("Cursor Changed: Getting description\n");
		gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, 2, &description, -1);
		if(description!= NULL)
		{
			GtkTextView *textview_description = GTK_TEXT_VIEW(lookup_widget(GTK_WIDGET(treeview), "textview_Description"));
			GtkTextBuffer* textbuffer = gtk_text_view_get_buffer  (textview_description);
            printf("Cursor Changed: Importing HTML, Description(%s)\n", description);
            import_XHTML (textbuffer, description);
		}
		else
		{
		}
	}
}
